@@ -101,10 +101,13 @@ choose_dir <- function(default = "", sep = c("/", "\\"))
 #' @importFrom assertive.reflection is_windows
 #' @examples 
 #' \donttest{
-#' choose_files()
-#' if(assertive.reflection::is_windows())
+#' if(interactive())
 #' {
-#'   choose_dir()
+#'   choose_files()
+#'   if(assertive.reflection::is_windows())
+#'   {
+#'     choose_dir()
+#'   }
 #' }
 #' }
 #' @export
@@ -159,38 +162,55 @@ choose_files <- function(default = "", multi = FALSE, sep = c("/", "\\"))
   standardize_path(x, sep, include_names = FALSE)
 }
 
-#' Create or remove directories
+#' Create or remove files and directories
 #' 
-#' A vectorized version of \code{\link[base]{dir.create}}, and 
-#' \code{\link[base]{unlink}} with more convenient defaults.
+#' A vectorized version of \code{\link[base]{dir.create}}, and
+#' \code{\link[base]{file.create}} and \code{\link[base]{unlink}} with more 
+#' convenient defaults.
 #' @param x A character vector of paths of directories to create/remove. 
 #' For \code{create_dirs}, it defaults to a directory inside \code{tempdir()}.
 #' @return A logical vector of successes of failures.
-#' @note \code{\link[base]{unlink}}, and consequently \code{remove_dirs},
-#' sometimes fails to remove empty directories.
-#' See \url{https://bugs.r-project.org/bugzilla3/show_bug.cgi?id=16287}.
+#' @note \code{create_dirs} will only attempt to create directories that don't 
+#' already exist.
 #' @seealso \code{\link[base]{dir.create}}, \code{\link[base]{unlink}}
 #' @examples
 #' \donttest{
-#' dirs <- file.path(temp_dir(), c("foo", "bar/baz"))
+#' dirs <- temp_dir(c("foo", "bar/baz"))
 #' create_dirs(dirs)
 #' 
 #' # Check this worked:
-#' assert_all_are_dirs(dirs)
+#' assertive.files::assert_all_are_dirs(dirs)
+#' 
+#' files <- temp_dir("blah/blah/blah", LETTERS)
+#' create_files(files)
+#' 
+#' assertive.files::assert_all_are_existing_files(files)
 #' 
 #' # Clean up
-#' remove_dirs(dirs)
+#' remove_dirs(temp_dir(c("foo", "bar", "blah")))
 #' }
 #' @importFrom stats setNames
 #' @export
 create_dirs <- function(x = temp_file(pattern = "dir"))
 {
-  vapply(
-    setNames(x, x),
+  doesnt_yet_exist <- !file.exists(x)
+  yn <- setNames(logical(length(x)), x)
+  yn[doesnt_yet_exist] <- vapply(
+    x[doesnt_yet_exist],
     dir.create,
-    logical(1),
+    logical(1), 
     recursive = TRUE
   )
+  yn
+}
+
+#' @rdname create_dirs
+#' @export
+create_files <- function(x = temp_file())
+{
+  dirs <- unique(dirname(standardize_path(x)))
+  create_dirs(dirs)
+  setNames(file.create(x), x)
 }
 
 #' Make a path suitable for cygwin
@@ -746,6 +766,26 @@ replace_extension <- function(x = dir(), new_extension, include_dir = NA)
   )
 }
 
+#' Get the RStudio project directory
+#' 
+#' Gets the current RStudio project directory.
+#' @param sep String separator between directory levels in the output.
+#' @return A string giving the path to the current RStudio project directory,
+#' or character vector with length zero if you are running RStudio without a
+#' project open.
+#' @note This only works when your IDE is RStudio. Otherwise an error is thrown.
+#' @examples 
+#' assertive.base::dont_stop(rstudio_project_dir())
+#' @importFrom assertive.reflection assert_is_rstudio
+#' @export
+rstudio_project_dir <- function(sep = c("/", "\\"))
+{
+  assert_is_rstudio()
+  e <- as.environment("tools:rstudio")
+  x <- e$.rs.getProjectDirectory()
+  standardize_path(x, sep = sep, include_names = FALSE)
+}
+
 #' Split a path into directory components
 #' 
 #' Splits a character vector of paths into directory components.  The opposite  
@@ -968,28 +1008,44 @@ system_file <- function(..., package = "base", library_location = NULL,
   ifelse(nzchar(paths), paths, NA_character_)
 }
 
-#' Create a temp file/dir
+#' Return paths to files or dirs within the temp dir
 #' 
-#' Wrappers to \code{tempdir} and \code{tempfile} that return standardized 
-#' paths.
-#' @param ... Passed to \code{tempfile}
+#' Vectorized wrappers to \code{tempdir} and \code{tempfile} that return  
+#' standardized paths.
+#' @param ... Character vectors of further directories within the temp 
+#' directory. Passed to \code{\link[base]{file.path}}.
+#' @param pattern Character vector of prefixes for the temp file name. 
+#' Passed to \code{\link[base]{tempfile}}.
+#' @param fileext Character vector of file extensions for the temp file. Passed 
+#' to \code{\link[base]{tempfile}}.
 #' @param sep String separator between directory levels in the output.
 #' @return For \code{temp_file} a character vector giving the names of possible 
 #' (temporary) files. Note that no files are generated by \code{temp_file}.
 #' For \code{temp_dir}, the path of the per-session temporary directory.
 #' @seealso \code{\link[base]{tempdir}}
 #' @examples
-#' temp_dir()
-#' temp_file()
+#' temp_dir(c("foo", "bar/baz"))
+#' temp_file(c("foo", "bar/baz"), fileext = c(".txt", ".R"))
 #' @export
-temp_dir <- function(sep = c("/", "\\"))
+temp_dir <- function(..., sep = c("/", "\\"))
 {
-  standardize_path(tempdir(), sep = sep, include_names = FALSE)
+  sep <- match.arg(sep)
+  tmp <- file.path(tempdir(), ..., fsep = sep)
+  standardize_path(tmp, sep = sep, include_names = FALSE)
 }
 
 #' @rdname temp_dir
 #' @export
-temp_file <- function(..., sep = c("/", "\\"))
+temp_file <- function(..., pattern = "file", fileext = "", sep = c("/", "\\"))
 {
-  standardize_path(tempfile(...), sep = sep, include_names = FALSE)
+  x <- temp_dir(...)
+  pattern <- rep_len(pattern, length(x))
+  fileext <- rep_len(fileext, length(x))
+  tmp <- mapply(
+    tempfile, 
+    x,
+    pattern = pattern, 
+    fileext = fileext
+  )
+  standardize_path(tmp, sep = sep, include_names = FALSE)
 }
